// sky light marker at 2-sided lines
class SkyLight : Actor
{
	Default
	{
		Scale 0.5;
		+FLOAT
		+NOGRAVITY
	}
	
	States
	{
	Spawn:
		BAL1 A -1;
		Stop;
	}
}

// light sensor marker at 2-sided lines
class LightSensor : Actor
{
	Default
	{
		Scale 0.5;
		+FLOAT
		+NOGRAVITY
	}
	
	int getLight(double dist)
	{
		int light = 96;
		if (CheckProximity("SkyLight", dist, 1, CPXF_CHECKSIGHT | CPXF_CLOSEST))
		{
			light = clamp(256.0 / dist * 255 + 32, 64, 255);
		}
		return light;
	}

	States
	{
	Spawn:
		PVIS AAAAAAAAAAA 5 Bright
		{
			int minlight = 96;
			int light = getLight(256.0);
			if (light == minlight)
			{
				light = getLight(512.0);
				if (light == minlight)
				{
					light = getLight(768.0);
					if (light == minlight)
					{
						light = getLight(1024.0);
					}
				}
			}
			Level.Sectors[args[0]].SetLightLevel(light);
		}
		Loop;
	}
}

// relighting event handler

class relighting_EventHandler : EventHandler
{
	override void WorldLoaded(WorldEvent e)
	{
		int FLOOR = 0;
		int CEILING = 1;

		// get this from the mapinfo
		Texman texture;
		textureid skyid = texture.CheckForTexture("F_SKY1", Texman.TYPE_ANY, Texman.TRYANY | Texman.OVERRIDABLE);

		for(int i = 0; i < Level.Sectors.Size(); i++)
		{
			Sector sec = Level.Sectors[i];
			SecPlane floorplane = sec.FloorPlane;
			SecPlane ceilingplane = sec.CeilingPlane;

			string cname = "LightSensor";
			textureid ceilingid = sec.GetTexture(CEILING);
			if (ceilingid == skyid)
			{
				sec.SetLightLevel(255);
				cname = "SkyLight";
			}
			else
			{
				sec.SetLightLevel(64);
			}


			for (int ii = 0; ii < sec.Lines.Size(); ii++)
			{
				Line lin = sec.Lines[ii];

				int f = lin.flags;
				if (f & Line.ML_TWOSIDED)
				{
					double x = (lin.V1.P.x + lin.V2.P.x) * 0.5;
					double y = (lin.V1.P.y + lin.V2.P.y) * 0.5;
					double z = (floorplane.ZAtPoint((x,y)) + ceilingplane.ZAtPoint((x,y))) * 0.5;

					Actor ptr = Actor.Spawn(cname, (x,y,z));
					if (ptr)
					{
						ptr.args[0] = i;
						ptr.args[1] = sec.lightlevel;
					}
				}
			}
		}
	}
}